import os

import hconf


class Configuration(hconf.ConfigManager):
    DEFAULT_SERVICES = ['ec2', 's3', 'rds']

    def process_list(self, data):
        if type(data) == list:
            return data
        return data.split(',')

    def __init__(self, session):
        self._session = session
        super().__init__(
            {'name': 'regions', 'default': [],
             'required': False, 'cast': self.process_list,
             'description': 'List of regions to check tags in'},
            {'name': 'services', 'default': [],
             'required': False, 'cast': self.process_list,
             'description': 'List of Services to check tags on'},
            {'name': 'required-tags',
             'required': True, 'cast': self.process_list,
             'description': 'List of tags which must exist'},
            {'name': 'dynamodb-table-name',
             'required': True,
             'description': 'Name of DynamoDB table to write tagging proposals to'},
            {'name': 'dynamodb-table-region',
             'required': True,
             'description': 'Region the DynamoDB table exists in'}
        )

        self.registerParser(hconf.Subparsers.YAML(
            filename="config.yml", filepath=os.path.expanduser('~/.config/tagaday/')
        ))

        self.registerParser(hconf.Subparsers.Cmdline(
            'Tool to work through AWS and tag instances')
        )

    def parse(self):
        cfg = super().parse()
        self._process_defaults(cfg)
        return cfg

    def _process_defaults(self, cfg):
        if len(cfg.regions) == 0:
            cfg.regions = [region['RegionName'] for region in
                           self._session().client('ec2').describe_regions()['Regions']]



from prompt_toolkit.validation import Validator, ValidationError


class NumberRangeValidator(Validator):
    def __init__(self, range=None):
        self._range = range

    def validate(self, document):
        text = document.text

        if text and not text.isdigit():
            for i, c in enumerate(text):
                if not c.isdigit():
                    break
            raise ValidationError(
                message='Please enter a number', cursor_position=i)



from tag_a_day.services.service import Service


class EC2TagHandler(Service):
    name = 'ec2_instance'
    missing_tags_text = "This instance is missing '{0}' in its tags."

    def resources(self, session):
        return session.resource('ec2').instances.all()

    def handler(self, instance, expected_tags, region, session, cache, proposals):
        # Get VPC info
        _, vpc_name, vpc = self._get_vpc_info(
            vpc_id=instance.vpc_id,
            region=region
        )

        # Build table for displaying instance information
        evaluated_tags = self._progress.evaluated_tags(instance.id)
        instance_info, missing_tags = \
            self._build_tag_sets(expected_tags, evaluated_tags, instance.tags)

        if any(missing_tags):
            self._print_table(
                ("Vpc", vpc_name),
                ("VpcID", vpc.id),
                ("InstanceId", instance.id),
                *instance_info
            )

            if self._progress.has_finished(instance.id, expected_tags):
                self._skip(instance.id)
                return

            tag_prompt = self._build_tag_prompt(missing_tags)
            for tag_key in missing_tags:
                yield {
                    'resource_id': instance.id,
                    'tag_key': tag_key,
                    'tag_value': tag_prompt(tag_key),
                }



from pkg_resources import iter_entry_points


class Services(object):
    def __init__(self, session, services, cache, proposals):

        init_params = {
            'session': session, 'table': proposals,
            'cache': cache
        }
        self._mapping = list(map(
            lambda ep: ep.load()(**init_params).load(),
            filter(
                lambda ep: ep.name in services,
                iter_entry_points(group='tag_a_day.tag_handlers', name=None)
            )
        ))

        self._current = 0

    def __iter__(self):
        self._current = 0
        return self

    def __next__(self):
        if self._current < len(self._mapping):
            self._current += 1
            return self._mapping[self._current - 1]
        else:
            raise StopIteration()
